//----------------------------------------------------------------------------------------------
//
// State - Navigate Kill Monsters
//
//----------------------------------------------------------------------------------------------
//
// Running around kill monsters close to the line between my current maploc and next
// waypoint maplocin, also attack monsters that has done melee damage to me?
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2003-03-18   Initial version.
//
//----------------------------------------------------------------------------------------------
//
// FIXME:
//  - Target priority, just like in TargetFinder state
//  - Geometry functions to common file
//  - Lots of strange bugs, read the fixme's
//  - Attack stuff that is hitting me with melee or missile (not spells)
//  - Sometimes just stops killing?  Not sure why, but will start if i pause and unpause so
//    probably has something to do with the KillAction getting added without pausing the
//    navigate action.
//
//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters()
{
  // Member variables
  this.mSubStates = new Array();
  this.mRouteAction = "";

  // Member functions
  this.EnterState = State_NavigateKillMonsters_EnterState;
  this.ExitState = State_NavigateKillMonsters_ExitState;
  this.Execute = State_NavigateKillMonsters_Execute;
  this.SetRouteAction = State_NavigateKillMonsters_SetRouteAction;
  this.SetKillAction = State_NavigateKillMonsters_SetKillAction;
  this.OnActionFailed = State_NavigateKillMonsters_OnActionFailed;
  this.SetTarget = State_NavigateKillMonsters_SetTarget;
  
  this.OnTimer = State_NavigateKillMonsters_OnTimer;
}

//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters_EnterState()
{
  gLog.TraceLine( "State.NavigateKillMonsters.EnterState", ">> State.NavigateKillMonsters.EnterState" );

  this.Inherited_EnterState();
  gTimer.AddTimeout( "NavKillTimer", 200, this, true );
  this.mActionManager.AddAction( this.mRouteAction );

  gLog.TraceLine( "Lib.State.NavigateKillMonsters.EnterState", "<< State.NavigateKillMonsters.EnterState" );
}

//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters_ExitState()
{
  gLog.TraceLine( "State.NavigateKillMonsters.ExitState", ">> State.NavigateKillMonsters.ExitState" );

  gTimer.RemoveTimeout( "NavKillTimer" );
  this.Inherited_ExitState();

  gLog.TraceLine( "State.NavigateKillMonsters.ExitState", "<< State.NavigateKillMonsters.ExitState" );
}

//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters_Execute()
{
  gLog.TraceLine( "State.NavigateKillMonsters.Execute", ">> State.NavigateKillMonsters.Execute" );

  while( !this.mStateMachine.ShouldExitState() )
  {

    // Make sure it checks for targets before resuming run
    this.OnTimer( null );
    
    this.mActionManager.ExecuteAction();
    skapi.WaitEvent( 0 );

    if( !this.mActionManager.HasActiveAction( this.mRouteAction ) )
    {
      this.mStateMachine.ChangeState( "Success" );    
    }

  }

  gLog.TraceLine( "State.NavigateKillMonsters.Execute", "<< State.NavigateKillMonsters.Execute" );
}

//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters_OnTimer( pData )
{
  var vRoute;
  
  vRoute = this.mActionManager.GetAction( this.mRouteAction ).mRoute;

  if( vRoute == null )
  {
  	return;
  }

  if( vRoute.iwaypt == 0 )
  {
    return;
  }
  
  if( vRoute.iwaypt == vRoute.cwaypt )
  {
    return;
  }
  
  if( this.mActionManager.HasActiveAction( this.mKillAction ) )
  {
    return;
  }
  
  this.SetTarget( vRoute );
}

//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters_SetTarget( pRoute )
{
  gLog.TraceLine( "State.NavigateKillMonsters.SetTarget", ">> State.NavigateKillMonsters.SetTarget" );

  gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Current Waypoint index: "+pRoute.iwaypt );
  gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Prev Waypt: "+pRoute.rgwaypt( pRoute.iwaypt-1 ).maploc+", Next Waypoint: "+ pRoute.rgwaypt( pRoute.iwaypt ).maploc );

  var vAllTargets = skapi.coacoNew();
  var vFilter = skapi.acfNew();
  var vTarget;

  vFilter.ocm = ocmMonster;
  var vTargets = vFilter.CoacoGet();
  
  var enu = new Enumerator( vTargets );

  if( enu == null )
  {
    // FIXME:
	//  - Still crashbugs, workaround....
    gLog.TraceLine( "State.NavigateKillMonsters.SetTarget", "<< State.NavigateKillMonsters.SetTarget" );
    return;
  }

  // FIXME:
  //  - Still crashbugs, workaround....  Division by zero causes it?
  
  try
  {  

    for( ; !enu.atEnd(); enu.moveNext() )
    {
      vTarget = enu.item();
    
      if( gPlayer.MapLoc().Dist2DToMaploc( vTarget.maploc ) < 0.1 )
      {
        gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Possible target: "+AcoToString( vTarget )+", Loc: "+vTarget.maploc );
		    
		  
        // FIXME:
	    //  - Holy crap, it can't really be this difficult
		    
	    // Calculate normalized vector of line
	    var vP1 = pRoute.rgwaypt( pRoute.iwaypt-1 ).maploc;
	    var vP2 = pRoute.rgwaypt( pRoute.iwaypt ).maploc;
	    var vP = vTarget.maploc;
		
	    // FIXME:
	    //  - I really don't understand this, but i hope it helps with the crash bug
		
	    if( vP1 == null || vP2 == null )
	    {
	      gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Unknown route problems with coordinates" );
	      break;
	    }
		
	    var vVectorX = vP2.lng-vP1.lng;
	    var vVectorY = vP2.lat-vP1.lat;
	    var vVectorLen = Math.sqrt( vVectorX*vVectorX+vVectorY*vVectorY );
	    vVectorX = vVectorX/vVectorLen;
	    vVectorY = vVectorY/vVectorLen;
		
	    // Distance from point to line    
	    var vXx0 = vP.lng-vP2.lng;
	    var vYy0 = vP.lat-vP2.lat;
		    
	    var vDist = Math.sqrt( vXx0*vXx0+vYy0*vYy0-( vVectorX*vXx0+vVectorY*vYy0)*( vVectorX*vXx0+vVectorY*vYy0) );
		
	    gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Distance: "+vDist );
	    gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Dist2: "+vP2.Dist2DToMaploc( vP ) );
		
	    if( vDist > vP2.Dist2DToMaploc( vP ) )
	    {
	      gLog.ErrorLine( "State.NavigateKillMonsters.SetTarget", "Distance to line is greater than distance to point!" );
	    }
		
	    // Project point on line and make sure it is between P1 and P2 (a bit past P2 is allowed but not too much)
		
	    var vK = ( vP2.lat-vP1.lat )/( vP2.lng-vP1.lng );
	    var vM = vP1.lat-vK*vP1.lng;
		    
	    var vKinv = -1/vK;
	    var vMinv = vP.lat-vKinv*vP.lng;
		    
	    var vProjX = (vM-vMinv)/(vKinv-vK);
	    var vProjY = vK*vProjX+vM;
		    
	    var vProj = skapi.MaplocFromLatLng( vProjY, vProjX, 0, 0 )
		
	    gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Projection: "+vProj );
	    gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Distance P1->P2:"+vP1.Dist2DToMaploc( vP2 ) );
		gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Distance P1->PP:"+vP1.Dist2DToMaploc( vProj ) );
		gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Distance P2->P1:"+vP2.Dist2DToMaploc( vP1 ) );
		gLog.DebugLine( "State.NavigateKillMonsters.SetTarget", "Distance P2->PP:"+vP2.Dist2DToMaploc( vProj ) );
		
		if( vP1.Dist2DToMaploc( vP2 )+0.01 > vP1.Dist2DToMaploc( vProj ) && vP2.Dist2DToMaploc( vP1 ) > vP2.Dist2DToMaploc( vProj ) )
		{
		  if( vDist < 0.01 )
		  {
	        vAllTargets.AddAco( vTarget );
          }
        }
		  
      }
	
    }
  }
  catch( e )
  {
    gLog.ErrorLine( "State.NavigateKillMonsters.SetTarget", "NavigateKillMonsters.SetTarget exception caught and ignored" );
    gLog.TraceLine( "State.NavigateKillMonsters.SetTarget", "<< State.NavigateKillMonsters.SetTarget" );
    return;
  }


  vTarget = gTargetManager.SetTarget( vAllTargets );
  if( vTarget != null )
  {
    this.mActionManager.GetAction( this.mKillAction ).SetTarget( vTarget );
    this.mActionManager.AddAction( this.mKillAction );
  }

  gLog.TraceLine( "State.NavigateKillMonsters.SetTarget", "<< State.NavigateKillMonsters.SetTarget" );
}

//----------------------------------------------------------------------------------------------
  
function State_NavigateKillMonsters_SetKillAction( pKillAction )
{
  this.mKillAction = pKillAction
}

//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters_SetRouteAction( pRouteAction )
{
  this.mRouteAction = pRouteAction
}

//----------------------------------------------------------------------------------------------

function State_NavigateKillMonsters_OnActionFailed( pSource, pAction, pReason )
{
  // Do stuff here
}

//----------------------------------------------------------------------------------------------
