Option Explicit
Class WayptKillTargetCls
' Normal waypoint class with no arrival action.


'---------------------------------------
' Type-specific properties and methods:
'---------------------------------------

Public maploc			' The location of this waypoint.
Public distArrive		' The arrival distance of this waypoint.

' Initialize this waypoint from the given parameters.
Public Function Init(maplocA, distArriveA)
	Call TraceEnterEx("WayptKillTarget.Init", SzTraceMaploc(maplocA), Xnav.tef)
	
	Set maploc = maplocA
	distArrive = distArriveA
	If distArrive <= 0 Then
	'	Default distArrive to some reasonable value.
		If fWalk Then
			distArrive = 1.0 * Xnav.distMeter
		Else
			distArrive = 2.0 * Xnav.distMeter
		End If
	End If
	
	Call TraceExit("WayptKillTarget.Init", "")
End Function


'----------------------------------
' Required properties and methods:
'----------------------------------

' The XML tag name of this waypoint type.
Public Property Get szTag()
	szTag = "KillTarget"
End Property

' True if this leg should be walked, false if it should be run.
' Xnav handles initialization of this property, but each waypoint class must define it.
Public fWalk

' The maploc Xnav should steer toward.
Public Property Get maplocIn()
	Set maplocIn = maploc
End Property

' The maploc we're at after the arrival action has taken place.
' For waypoints with no arrival action, this is the same as maplocIn.
Public Property Get maplocOut()
	Set maplocOut = maploc
End Property

' Since distArrive is configurable, I've listed it above under "Type-specific 
' properties and methods".

' The estimated time in seconds needed for this waypoint's arrival action.
' For waypoints with no arrival action, this is zero.
' Used by xroute.csecTravel.
Public Property Get csecArrive()
	csecArrive = 0
End Property

' Perform this waypoint's arrival action.
Public Function Arrive()
	Call TraceEnterEx("WayptKillTarget.Arrive", SzTraceMaploc(maploc), Xnav.tef)

	
'	Dim acoLever : Set acoLever = AcoFindLever()
'	
'	Call acoLever.Use()
'
'    Call skapi.WaitEvent( 500, wemFulltimeout )

	Call TraceExit("WayptKillTarget.Arrive", "")
End Function

' Generate a new waypoint representing the reversal of this waypoint.
' Used by xroute.XrouteReverse.
Public Function WayptReverse()
	Call TraceEnterEx("WayptKillTarget.WayptReverse", SzTraceMaploc(maploc), Xnav.tef)
	
'	Normal waypoints are the same coming and going.  Just clone it.
	Set WayptReverse = WayptKillTargetNew()
	Call WayptReverse.Init(maploc, distArrive)
	
	Call TraceExit("WayptKillTarget.WayptReverse", "")
End Function

' Initialize this waypoint from skapi.maplocCur and, if relevant, skapi.acoSelected.
' Used by XrouteEdit.
Public Function FromMaplocCur()
	Call TraceEnterEx("WayptKillTarget.FromMaplocCur", "", Xnav.tef)
	
	Call Init(skapi.maplocCur, -1)
	
	Call TraceExit("WayptKillTarget.FromMaplocCur", "")
End Function

' Initialize this waypoint from an XML element.
Public Function FromXml(elem)
	Call TraceEnterEx("WayptKillTarget.FromXml", SzQuote(elem.xml), Xnav.tef)
	
	Dim distArrive : distArrive = FloatGetAttr(elem, "distArrive", -1)
	
	Call Init(Xnav.MaplocOfElem(elem), distArrive)
	
	Call TraceExit("WayptKillTarget.FromXml", "")
End Function

' Save type-specific waypoint parameters to the given XML element.
' If fVerbose is True, all waypoint parameters should be made explicit in the XML.  
' (This is needed by XrouteEdit.)  If fVerbose is False, parameters with default 
' values may be omitted.
Public Function ToXml(elem, fVerbose)
	Call TraceEnterEx("WayptKillTarget.ToXml", CStr(fVerbose), Xnav.tef)
	
	Call Xnav.SetElemMaploc(elem, maploc, True, False)
	Call SetFloatAttr(elem, "distArrive", distArrive, 4)
	
	Call TraceExit("WayptKillTarget.ToXml", "")
End Function


' Look for a KillTarget near the expected coordinates.
Private Function AcoFindKillTarget()
	Call TraceEnter("WayptKillTarget.AcoFindKillTarget", SzTraceMaploc(maploc))
	
	Dim acf : Set acf = skapi.AcfNew()
	Set acf.maploc = skapi.MaplocFromLatLng(maploc.lat, maploc.lng, _
	  skapi.maplocCur.z)
	acf.szName = "KillTarget"
	acf.distMax = 0.1
	
	Set AcoFindKillTarget = acf.AcoGetSorted(skapi.sortbyDist)
	
	Call TraceExit("WayptKillTarget.AcoFindKillTarget", SzTraceAco(AcoFindKillTarget))
End Function


'-----------
' Privates:
'-----------

Private Sub Class_Initialize()
	
	Set maploc = Nothing
	distArrive = Xnav.distMeter
	
End Sub


End Class


' Instantiator function.
Public Function WayptKillTargetNew()
	Set WayptKillTargetNew = New WayptKillTargetCls
End Function

' Register the waypoint type.
Call Xnav.RegisterType("KillTarget", "WayptKillTargetNew")


