//----------------------------------------------------------------------------------------------
//
// Action - Equip Item
//
//----------------------------------------------------------------------------------------------
//
// Equips an item (only tested for arrows)
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-09-12   Initial version.
//
//----------------------------------------------------------------------------------------------
//
// FIXME:
//  - Timeout different for combat/peace mode?
//  - No item found, action failure?
//
//----------------------------------------------------------------------------------------------

function Action_EquipItem()
{
  // Member variables
  this.mIsWaiting = false;

  // Member fuctions
  this.StartAction = Action_EquipItem_StartAction;
  this.ResumeAction = Action_EquipItem_ResumeAction;
  this.SetItemName = Action_EquipItem_SetItemName;
  this.SetDelay = Action_EquipItem_SetDelay;
  this.Equip = Action_EquipItem_Equip;

  // Event handlers
  this.OnRemoveFromInventory = Action_EquipItem_OnRemoveFromInventory;
}

//----------------------------------------------------------------------------------------------

function Action_EquipItem_StartAction()
{
  this.Inherited_StartAction();
  this.Equip();
}

//----------------------------------------------------------------------------------------------

function Action_EquipItem_ResumeAction()
{
  this.Inherited_ResumeAction();
  this.Equip();
}

//----------------------------------------------------------------------------------------------

function Action_EquipItem_Equip()
{
  gLog.TraceLine( "Lib.Action.EquipItem.Equip", ">> Action.EquipItem.Equip" );

  var vFilter = skapi.acfNew();
  vFilter.szName = this.mItemName;
  vFilter.olc = olcInventory;

  var vAco = vFilter.AcoGet();

  if( vAco == null )
  {
    // FIXME: Action failure
    return;
  }

  skapi.AddHandler( evidOnRemoveFromInventory, this );
  this.mIsWaiting = true;
  vAco.Use();

  for( var i = 0; i < 20; i++ )
  {
    skapi.WaitEvent( 100, wemFullTimeout );
    if( !this.mIsWaiting )
    {
      break;
    }
  }

  skapi.WaitEvent( this.mDelay, wemFullTimeout );

  skapi.RemoveHandler( evidOnRemoveFromInventory, this );

  gLog.TraceLine( "Lib.Action.EquipItem.Equip", "<< Action.EquipItem.Equip" );
}

//----------------------------------------------------------------------------------------------

function Action_EquipItem_SetItemName( pName )
{
  this.mItemName = pName;
}

//----------------------------------------------------------------------------------------------

function Action_EquipItem_SetDelay( pDelay )
{
  this.mDelay = pDelay;
}

//----------------------------------------------------------------------------------------------

function Action_EquipItem_OnRemoveFromInventory( aco, acoContainer, iitem )
{
  gLog.TraceLine( "Lib.Action.EquipItem.OnRemoveFromInventory", ">> EVENT: Action.EquipItem.OnRemoveFromInventory" );
  this.mIsWaiting = false;
  gLog.TraceLine( "Lib.Action.EquipItem.OnRemoveFromInventory", "<< EVENT: Action.EquipItem.OnRemoveFromInventory" );
}

//----------------------------------------------------------------------------------------------

