//----------------------------------------------------------------------------------------------
//
// Action - Kill Target
//
//----------------------------------------------------------------------------------------------
//
// Tries to kill the selected target
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-09-17   Initial version.
//
//----------------------------------------------------------------------------------------------

function Action_KillTarget()
{
  // Member variables
  this.mCurrentTarget = null;
  this.mNofMisses = 0;

  // Member functions
  this.StartAction = Action_KillTarget_StartAction;
  this.PauseAction = Action_KillTarget_PauseAction;
  this.ResumeAction = Action_KillTarget_ResumeAction;
  this.Attack = Action_KillTarget_Attack;
  this.WaitUntilDead = Action_KillTarget_WaitUntilDead;
  this.OnMeleeDamageOther = Action_KillTarget_OnMeleeDamageOther;
  this.OnChatServer = Action_KillTarget_OnChatServer;
  this.SetWeaponName = Action_KillTarget_SetWeaponName;
  this.SetWieldAction = Action_KillTarget_SetWieldAction;
  this.SetTarget = Action_KillTarget_SetTarget;
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_StartAction()
{
  gLog.TraceLine( "Lib.Action.KillTarget.StartAction", ">> Action.KillTarget.StartAction" );

  this.Inherited_StartAction();

  gLog.DebugLine( "Lib.Action.KillTarget.StartAction", "Current target: "+AcoToString( this.mCurrentTarget ) );

  this.Attack();

  gLog.TraceLine( "Lib.Action.KillTarget.StartAction", "<< Action.KillTarget.StartAction" );
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_ResumeAction()
{
  gLog.TraceLine( "Lib.Action.KillTarget.ResumeAction", ">> Action.KillTarget.ResumeAction" );

  this.Inherited_ResumeAction();

  gLog.DebugLine( "Lib.Action.KillTarget.ResumeAction", "Selected: "+AcoToString( skapi.acoSelected )+"  Wanted: "+AcoToString( this.mCurrentTarget ) );

  this.Attack();

  gLog.TraceLine( "Lib.Action.KillTarget.ResumeAction", "<< Action.KillTarget.ResumeAction" );
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_PauseAction()
{
  if( skapi.fCombatMode )
  {
//    gInteract.Keys( cmidWalkBackwards );
    gInteract.Keys( "y" );
    skapi.WaitEvent( 300, wemFullTimeout );
  }
  this.Inherited_PauseAction();
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_Attack()
{
  // FIXME:
  //   - Archer only with preset aim...  


  if( !this.mCurrentTarget.fExists )
  {
    gLog.DebugLine( "Lib.Action.KillTarget.Attack", "Target no longer valid" );
    return;
  }

  if( this.mCurrentTarget != skapi.acoSelected )
  {
    skapi.SelectAco( this.mCurrentTarget );
  }

  if( skapi.AcoFromEqm( eqmAmmo ) == null )
  {
    gLog.DebugLine( "Lib.Action.KillTarget.Attack", "Adding reload arrows action" );
    this.mActionManager.AddAction( "EquipArrows" );
    if( this.mIsPaused )
    {
      return;
    }
  }

  if( skapi.AcoFromEqm( eqmWeapons ) == null )
  {
    gLog.DebugLine( "Lib.Action.KillTarget.Attack", "Adding wield action" );
    this.mActionManager.AddAction( this.mWieldAction );
    if( this.mIsPaused )
    {
      return;
    }
  }

  if( skapi.AcoFromEqm( eqmWeapons ).szName != this.mWeaponName )
  {
    gLog.DebugLine( "Lib.Action.KillTarget.Attack", "Adding wield action" );
    this.mActionManager.AddAction( this.mWieldAction );
    if( this.mIsPaused )
    {
      return;
    }
  }

  if( !skapi.fCombatMode )
  {
    this.mActionManager.AddAction( "CombatMode" );
    if( this.mIsPaused )
    {
      return;
    }
  }

  // FIXME:
  //  - Delay needed?  I press attack too fast sometimes it seems!
  skapi.WaitEvent( 200, wemFullTimeout );

  this.mDamageTimer = skapi.TimerNew();
  this.mNofMisses = 0;

  skapi.AddHandler( evidOnMeleeDamageOther, this );
  skapi.AddHandler( evidOnChatServer, this );

  gInteract.Keys( cmidMediumAttack );

  // FIXME:
  //  - Turn to monster for faster attack?
  
  this.WaitUntilDead();

  skapi.RemoveHandler( evidOnMeleeDamageOther, this );
  skapi.RemoveHandler( evidOnChatServer, this );
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_WaitUntilDead()
{
  gLog.TraceLine( "Lib.Action.KillTarget.WaitUntilDead", ">> Action.KillTarget.WaitUntilDead" );

  // FIXME:
  //   - Also give up on target if i miss too many times in a row, and give that target lower priority?

  while( true )
  {
    if( skapi.acoSelected == null )
    {
      gTargetManager.LowerPriority( this.mCurrentTarget );
      gLog.DebugLine( "Lib.Action.KillTarget.WaitUntilDead", "Exiting KillTarget, Reason: Lost target" );
      break;
    }

    if( skapi.acoSelected != this.mCurrentTarget )
    {
      gLog.DebugLine( "Lib.Action.KillTarget.WaitUntilDead", "Exiting KillTarget, Reason: New target" );
      break;
    }

    if( !skapi.fCombatMode )
    {
      if( skapi.AcoFromEqm( eqmAmmo ) == null )
      {

	    skapi.WaitEvent( 1000, wemFullTimeout );
        this.mActionManager.AddAction( "EquipArrows" );
      }
    }

    if( this.mNofMisses > 3 )
    {
      gTargetManager.LowerPriority( this.mCurrentTarget );
      gLog.DebugLine( "Lib.Action.KillTarget.WaitUntilDead", "Exiting KillTarget, Reason: Nof misses" );
      gInteract.Keys( "y" );
      break;
    }

    if( this.mDamageTimer.cmsec > 5000 )
    {
      gInteract.Keys( cmidMediumAttack );
      this.mDamageTimer = skapi.TimerNew();
    }

    if( this.mIsPaused || this.mIsAborted )
    {
      gInteract.Keys( "y" );
      gLog.DebugLine( "Lib.Action.KillTarget.WaitUntilDead", "Exiting KillTarget, Reason: Paused" );
      break;
    }

    skapi.WaitEvent( 100 );
  }

  gLog.TraceLine( "Lib.Action.KillTarget.WaitUntilDead", "<< Action.KillTarget.WaitUntilDead" );
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_OnMeleeDamageOther( szName, dhealth, dmty )
{
  this.mDamageTimer = skapi.TimerNew();
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_OnChatServer( szMsg, cmc )
{
  if( szMsg == "Your missile attack hit the environment." || szMsg == "Your missile attack hit another target!" )
  {
    this.mDamageTimer = skapi.TimerNew();
    this.mNofMisses++;
  }
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_SetWeaponName( pWeaponName )
{
  this.mWeaponName = pWeaponName;
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_SetWieldAction( pWieldAction )
{
  this.mWieldAction = pWieldAction;
}

//----------------------------------------------------------------------------------------------

function Action_KillTarget_SetTarget( pTarget )
{
  this.mCurrentTarget = pTarget;
}

//----------------------------------------------------------------------------------------------
