//----------------------------------------------------------------------------------------------
//
// Action - Loot Corpse
//
//----------------------------------------------------------------------------------------------
//
// Loots an item from a corpse
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2003-02-09   Initial version.
//
//----------------------------------------------------------------------------------------------

function Action_LootCorpse()
{
  // Member variables
  this.mCurrentCorpse = null;
  this.mCurrentItem = null;
  this.mContents = null;
  this.mPickedUp = false;
  this.mOpened = false;
  
  this.mLootNames = new Array();
  this.mSalvageNames = new Array();

  // Member functions
  this.StartAction = Action_LootCorpse_StartAction;
  this.PauseAction = Action_LootCorpse_PauseAction;
  this.ResumeAction = Action_LootCorpse_ResumeAction;
  this.Loot = Action_LootCorpse_Loot;
  this.PickupItems = Action_LootCorpse_PickupItems;
  this.IsSalvage = Action_LootCorpse_IsSalvage;
  this.SetCorpse = Action_LootCorpse_SetCorpse;
  this.SetLootNames = Action_LootCorpse_SetLootNames;
  this.SetSalvageNames = Action_LootCorpse_SetSalvageNames;

  this.WaitUntilOpened = Action_LootCorpse_WaitUntilOpened;
  this.WaitUntilPickedUp = Action_LootCorpse_WaitUntilPickedUp;
  this.WaitContainerContents = Action_LootCorpse_WaitContainerContents;

  // Event listeners
  this.OnAddToInventory = Action_LootCorpse_OnAddToInventory;
  this.OnAdjustStack = Action_LootCorpse_OnAdjustStack
  this.OnOpenContainer = Action_LootCorpse_OnOpenContainer;
  this.OnObjectCreate = Action_LootCorpse_OnObjectCreate;
  this.OnActionComplete = Action_LootCorpse_OnActionComplete;
}
//----------------------------------------------------------------------------------------------

function Action_LootCorpse_StartAction()
{
  gLog.TraceLine( "Lib.Action.LootCorpse.StartAction", ">> Action.LootCorpse.StartAction" );

  this.Inherited_StartAction();

  gLog.DebugLine( "Lib.Action.LootCorpse.StartAction", "Current Corpse: "+AcoToString( this.mCurrentCorpse ) );

  this.mPickedUp = false;
  this.mOpened = false;
  if( this.mCurrentCorpse != null )
  {
    this.Loot();
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.StartAction", "<< Action.LootCorpse.StartAction" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_ResumeAction()
{
  gLog.TraceLine( "Lib.Action.LootCorpse.ResumeAction", ">> Action.LootCorpse.ResumeAction" );

  this.Inherited_ResumeAction();

  gLog.DebugLine( "Lib.Action.LootCorpse.ResumeAction", "Selected: "+AcoToString( skapi.acoSelected )+"  Wanted: "+AcoToString( this.mCurrentCorpse ) );

  if( this.mCurrentCorpse != null )
  {
    this.Loot();
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.ResumeAction", "<< Action.LootCorpse.ResumeAction" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_PauseAction()
{
  if( skapi.fCombatMode )
  {
    gInteract.Keys( cmidWalkBackwards );
  }
  this.Inherited_PauseAction();
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_Loot()
{
  gLog.TraceLine( "Lib.Action.LootCorpse.Loot", ">> Action.LootCorpse.Loot()" );

  if( skapi.fCombatMode )
  {
    this.mActionManager.AddAction( "PeaceMode" );
    if( this.mIsPaused )
    {
      gLog.TraceLine( "Lib.Action.LootCorpse.Loot", "<< Action.LootCorpse.Loot" );
      return;
    }
  }

  skapi.AddHandler( evidOnAddToInventory, this );
  skapi.AddHandler( evidOnAdjustStack, this );
  skapi.AddHandler( evidOnOpenContainer, this );
  skapi.AddHandler( evidOnObjectCreate, this );
  skapi.AddHandler( evidOnActionComplete, this );

  this.mCurrentCorpse.Use();
  if( this.WaitUntilOpened() )
  {
    this.PickupItems();
  }

  skapi.RemoveHandler( evidOnAddToInventory, this );
  skapi.RemoveHandler( evidOnAdjustStack, this );
  skapi.RemoveHandler( evidOnOpenContainer, this );
  skapi.RemoveHandler( evidOnObjectCreate, this );
  skapi.RemoveHandler( evidOnActionComplete, this );

  gLog.TraceLine( "Lib.Action.LootCorpse.Loot", "<< Action.LootCorpse.Loot" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_WaitUntilOpened()
{
  gLog.TraceLine( "Lib.Action.LootCorpse.WaitUntilOpened", ">> Action.LootCorpse.WaitUntilOpened()" );

  var vTimer = skapi.TimerNew();
  var vResult;

  while( true )
  {
    if( this.mOpened )
    {
      vResult = this.WaitContainerContents();
      break;
    }

    if( vTimer.cmsec > 10000 )
    {
      vResult = false;
      break;
    }

    if( this.mIsPaused )
    {
      vResult = false;
      break;
    }

    skapi.WaitEvent( 100 );
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.WaitUntilOpened", "<< Action.LootCorpse.WaitUntilOpened: "+vResult );
  return vResult;
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_WaitContainerContents()
{
  gLog.TraceLine( "Lib.Action.LootCorpse.WaitContainerContents", ">> Action.LootCorpse.WaitContainerContents()" );

  var vContents = this.mCurrentCorpse.coacoContents;
  
  if( vContents == null )
  {
    gLog.TraceLine( "Lib.Action.LootCorpse.WaitContainerContents", "<< Action.LootCorpse.WaitContainerContents: false" );
    return false;
  }
  
  var vEnum = new Enumerator( vContents );
  var vItem;

  this.mContents = skapi.CoacoNew();

  for( ; !vEnum.atEnd(); vEnum.moveNext() )
  {
    vItem = vEnum.item();
    if( vItem.szName == "" )
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitContainerContents", "New item found: "+AcoToString( vItem ) );
      this.mContents.AddAco( vItem );
    }
    else
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitContainerContents", "Old item found: "+AcoToString( vItem ) );
    }
  }

  // Wait for object info for contents of corpse
  var vTimer = skapi.TimerNew();
  while( true )
  {
  
    if( this.mContents == null )
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitContainerContents", "All item info received" );
      break;
    }
  
    if( this.mContents.Count == 0 )
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitContainerContents", "All item info received" );
      break;
    }

    if( vTimer.cmsec > 2000 )
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitContainerContents", "Timeout waiting for item info" );
      break;
    }

    if( this.mIsPaused || this.mIsAborted )
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitContainerContents", "paused or aborted" );
      break;
    }

    skapi.WaitEvent( 100 );
  }
  this.mContents = null;

  gLog.TraceLine( "Lib.Action.LootCorpse.WaitContainerContents", "<< Action.LootCorpse.WaitContainerContents: true" );
  return true;
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_PickupItems()
{
  gLog.TraceLine( "Lib.Action.LootCorpse.PickupItems", ">> Action.LootCorpse.PickupItems()" );

  var vFilter = skapi.acfNew();
  var i;
  
  gLog.DebugLine( "Lib.Action.LootCorpse.PickupItems", "Number of loot filters: "+this.mLootNames.length );
  
  for( i = 0; i < this.mLootNames.length; i++ )
  {
    vFilter.szName = this.mLootNames[ i ];

    gLog.DebugLine( "Lib.Action.LootCorpse.PickupItems", "Filtering contents for: "+vFilter.szName );

	  var vLoot = vFilter.AcoGet( this.mCurrentCorpse.coacoContents  );
	  while( vLoot != null )
	  {
		  gLog.DebugLine( "Lib.Action.LootCorpse.PickupItems", "Found loot item: "+AcoToString( vLoot ));
  	    
		  this.mCurrentItem = vLoot;
		  vLoot.Use();
		  if( !this.WaitUntilPickedUp() )
		  {
		    // Pickup timeout
		    break;
		  }

		  vLoot = vFilter.AcoGet( this.mCurrentCorpse.coacoContents  );
	  }
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.PickupItems", "<< Action.LootCorpse.PickupItems" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_WaitUntilPickedUp()
{
  gLog.TraceLine( "Lib.Action.LootCorpse.WaitUntilPickedUp", ">> Action.LootCorpse.WaitUntilPickedUp()" );

  var vResult;
  var vTimer = skapi.TimerNew();

  vResult = true;

  while( true )
  {
    if( this.mPickedUp )
    {
      gInteract.WaitActionComplete( 2000 );
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitUntilPickedUp", "Item pickedup" );
      break;
    }

    if( vTimer.cmsec > 10000 )
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitUntilPickedUp", "Timeout witing for pickup" );
      vResult = false;
      break;
    }

    if( this.mIsPaused || this.mIsAborted )
    {
      gLog.DebugLine( "Lib.Action.LootCorpse.WaitUntilPickedUp", "paused or aborted" );
      break;
    }

    skapi.WaitEvent( 100 );
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.WaitUntilPickedUp", "<< Action.LootCorpse.WaitUntilPickedUp: "+vResult );
  return vResult;
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_OnAddToInventory( pAco )
{
  gLog.TraceLine( "Lib.Action.LootCorpse.OnAddToInventory", ">> EVENT: Action.LootCorpse.OnAddToInventory( "+AcoToString( pAco )+" )" );

  var i;

  if( pAco == this.mCurrentItem )
  {
    this.mPickedUp = true;
    if( this.IsSalvage( pAco ) )
    {
      gSalvageManager.AddItem( pAco );
      if( !this.mActionManager.HasActiveAction( "Salvage" ) )
      {
        this.mActionManager.AddAction( "Salvage" );
      }
    }
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.OnAddToInventory", "<< EVENT: Action.LootCorpse.OnAddToInventory" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_OnAdjustStack( pAco, citemPrev )
{
  gLog.TraceLine( "Lib.Action.LootCorpse.OnAdjustStack", ">> EVENT: Action.LootCorpse.OnAdjustStack()" );

  this.mPickedUp = true;
  
  gLog.TraceLine( "Lib.Action.LootCorpse.OnAdjustStack", "<< EVENT: Action.LootCorpse.OnAdjustStack" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_OnOpenContainer( pAco )
{
  gLog.TraceLine( "Lib.Action.LootCorpse.OnOpenContainer", ">> EVENT: Action.LootCorpse.OnOpenContainer( "+AcoToString( pAco )+" )" );

  if( pAco == this.mCurrentCorpse )
  {
    gLog.TraceLine( "Lib.Action.LootCorpse.OnOpenContainer", "Nof items in corpse: "+pAco.citemContents );   
    this.mOpened = true;
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.OnOpenContainer", "<< EVENT: Action.LootCorpse.OnOpenContainer" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_OnObjectCreate( pAco )
{
  gLog.TraceLine( "Lib.Action.LootCorpse.OnObjectCreate", ">> EVENT: Action.LootCorpse.OnObjectCreate( "+AcoToString( pAco )+" )" );

  if( this.mContents != null )
  {
    if( this.mContents.FContainsAco( pAco ) )
    {
      this.mContents.RemoveAco( pAco );
    }
  }

  gLog.TraceLine( "Lib.Action.LootCorpse.OnObjectCreate", "<< EVENT: Action.LootCorpse.OnObjectCreate" );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_SetCorpse( pCorpse )
{
  this.mCurrentCorpse = pCorpse;
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_SetLootNames( pLootNames )
{
  this.mLootNames = this.mLootNames.concat( pLootNames.split( "," ) );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_SetSalvageNames( pSalvageNames )
{
  this.mLootNames = this.mLootNames.concat( pSalvageNames.split( "," ) );
  this.mSalvageNames = pSalvageNames.split( "," );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_OnActionComplete()
{
//  skapi.outputLine( "ActionComplete: "+skapi.arc );
}

//----------------------------------------------------------------------------------------------

function Action_LootCorpse_IsSalvage( pAco )
{
  var vFilter;
  var vItem;
  var i;
  
  vFilter = skapi.AcfNew();
  vItem = skapi.CoacoNew();
  vItem.AddAco( pAco );
  
  for( i = 0; i < this.mSalvageNames.length; i++ )
  {
    vFilter.szName = this.mSalvageNames[ i ];
    
    if( vFilter.AcoGet( vItem ) != null )
    {
      return true;
    }
  }
  return false;
}

//----------------------------------------------------------------------------------------------
