//----------------------------------------------------------------------------------------------
//
// Action - Loot Ground
//
//----------------------------------------------------------------------------------------------
//
// Loots an item from the ground
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-10-20   Initial version.
// 2.1   2003-03-13   Deprecated, loot no longer drops on ground.
//
//----------------------------------------------------------------------------------------------

function Action_LootGround()
{
  // Member variables
  this.mCurrentTarget = null;
  this.mPickedUp = false;

  // Member functions
  this.StartAction = Action_LootGround_StartAction;
  this.PauseAction = Action_LootGround_PauseAction;
  this.ResumeAction = Action_LootGround_ResumeAction;
  this.Loot = Action_LootGround_Loot;
  this.SetTarget = Action_LootGround_SetTarget;
  this.OnAddToInventory = Action_LootGround_OnAddToInventory;
  this.WaitUntilPickedUp = Action_LootGround_WaitUntilPickedUp;
}
//----------------------------------------------------------------------------------------------

function Action_LootGround_StartAction()
{
  gLog.TraceLine( "Lib.Action.LootGround.StartAction", ">> Action.LootGround.StartAction" );

  this.Inherited_StartAction();

  gLog.DebugLine( "Lib.Action.LootGround.StartAction", "Current target: "+AcoToString( this.mCurrentTarget ) );

  this.mPickedUp = false;
  if( this.mCurrentTarget != null )
  {
    this.Loot();
  }

  gLog.TraceLine( "Lib.Action.LootGround.StartAction", "<< Action.LootGround.StartAction" );
}

//----------------------------------------------------------------------------------------------

function Action_LootGround_ResumeAction()
{
  gLog.TraceLine( "Lib.Action.LootGround.ResumeAction", ">> Action.LootGround.ResumeAction" );

  this.Inherited_ResumeAction();

  gLog.DebugLine( "Lib.Action.LootGround.ResumeAction", "Selected: "+AcoToString( skapi.acoSelected )+"  Wanted: "+AcoToString( this.mCurrentTarget ) );

  if( this.mCurrentTarget != null )
  {
    this.Loot();
  }

  gLog.TraceLine( "Lib.Action.LootGround.ResumeAction", "<< Action.LootGround.ResumeAction" );
}

//----------------------------------------------------------------------------------------------

function Action_LootGround_PauseAction()
{
  if( skapi.fCombatMode )
  {
    gInteract.Keys( cmidWalkBackwards );
  }
  this.Inherited_PauseAction();
}

//----------------------------------------------------------------------------------------------

function Action_LootGround_Loot()
{
  if( skapi.fCombatMode )
  {
    this.mActionManager.AddAction( "PeaceMode" );
    if( this.mIsPaused )
    {
      return;
    }
  }

  skapi.AddHandler( evidOnAddToInventory, this );

  this.mCurrentTarget.Use();
  this.WaitUntilPickedUp();

  skapi.RemoveHandler( evidOnAddToInventory, this );
}

//----------------------------------------------------------------------------------------------

function Action_LootGround_WaitUntilPickedUp()
{
  var vTimer = skapi.TimerNew();

  while( true )
  {
    if( this.mPickedUp )
    {
      gInteract.WaitActionComplete( 2000 );
      break;
    }

    if( vTimer.cmsec > 10000 )
    {
      break;
    }

    if( this.mIsPaused )
    {
      break;
    }

    skapi.WaitEvent( 100 );
  }
}


//----------------------------------------------------------------------------------------------

function Action_LootGround_OnAddToInventory(aco)
{
  if( aco == this.mCurrentTarget )
  {
    this.mPickedUp = true;
  }
}

//----------------------------------------------------------------------------------------------

function Action_LootGround_SetTarget( pTarget )
{
  this.mCurrentTarget = pTarget;
}

//----------------------------------------------------------------------------------------------
