//----------------------------------------------------------------------------------------------
//
// Action - Navigate
//
//----------------------------------------------------------------------------------------------
//
// Standard action class for moving around, can presently only handle route files.
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-09-14   Initial version.
// 2.1   2003-03-15   Converted to use Xnav.
//
//----------------------------------------------------------------------------------------------
//
// FIXME:
//  - Doesn't get paused properly in some cases, no clue when, noticed it a bunch of times
//    in the NavigateKillMonsters state.
//
//----------------------------------------------------------------------------------------------

function Action_Navigate()
{
  // Member variables
  this.mRoutes = "I have no clue";
  this.mRoute = null;
  this.mRotueLegs = "I have no clue";
  this.mCurrentLeg = 0;

  // Member functions
  this.StartAction = Action_Navigate_StartAction;
  this.PauseAction = Action_Navigate_PauseAction;
  this.ResumeAction = Action_Navigate_ResumeAction;
  this.AbortAction = Action_Navigate_AbortAction;
  this.SetRouteFileName = Action_Navigate_SetRouteFileName;
  this.SetRouteLegs = Action_Navigate_SetRouteLegs;
  this.Navigate = Action_Navigate_Navigate;
  this.SetCombatMode = Action_Navigate_SetCombatMode;
  this.CombatMode = Action_Navigate_CombatMode;
  this.Turn = Action_Navigate_Turn;
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_StartAction()
{
  gLog.TraceLine( "Lib.Action.Navigate.StartAction", ">> Action.Navigate.StartAction()" );
  this.Inherited_StartAction();

  this.mCurrentLeg = 0;
  
  this.CombatMode();
  
  this.Navigate();
  
  gLog.TraceLine( "Lib.Action.Navigate.StartAction", "<< Action.Navigate.StartAction" );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_PauseAction()
{
  gLog.TraceLine( "Lib.Action.Navigate.PauseAction", ">> Action.Navigate.PauseAction()" );

  Xnav.StopNav();
  this.Inherited_PauseAction();

  gLog.TraceLine( "Lib.Action.Navigate.PauseAction", "<< Action.Navigate.PauseAction" );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_ResumeAction()
{
  gLog.TraceLine( "Lib.Action.Navigate.ResumeAction", ">> Action.Navigate.ResumeAction()" );

  this.Inherited_ResumeAction();
  
  this.CombatMode();
  
  this.Turn();
  if( this.mIsPaused || this.mIsAborted )
  {
    gLog.TraceLine( "Lib.Action.Navigate.ResumeAction", "<< Action.Navigate.ResumeAction" );
    return;
  }
  
  if( this.mRoute != null )
  {
	this.mRoute.ResumeRoute();
  }
  if( !this.mIsPaused && !this.mIsAborted )
  {
    this.mCurrentLeg = this.mCurrentLeg+1;
    this.Navigate();
  }
  else
  {
    this.mHeading = gPlayer.MapLoc().head;
  }

  gLog.TraceLine( "Lib.Action.Navigate.ResumeAction", "<< Action.Navigate.ResumeAction" );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_AbortAction()
{
  gLog.TraceLine( "Lib.Action.Navigate.AbortAction", ">> Action.Navigate.AbortAction()" );

  this.Inherited_AbortAction();
  Xnav.StopNav();

  gLog.TraceLine( "Lib.Action.Navigate.AbortAction", "<< Action.Navigate.AbortAction" );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_Navigate()
{
  gLog.TraceLine( "Lib.Action.Navigate.Navigate", ">> Action.Navigate.Navigate()" );
  
  if( this.mIsPaused )
  {
    this.mHeading = gPlayer.MapLoc().head;
    gLog.TraceLine( "Lib.Action.Navigate.Navigate", "<< Action.Navigate.Navigate" );
    return;
  }
  
  gLog.DebugLine( "Lib.Action.Navigate.Navigate", "Leg No: "+this.mCurrentLeg );

  while( this.mCurrentLeg < this.mRouteLegs.length )
  {
    gLog.DebugLine( "Lib.Action.Navigate.Navigate", "Leg: "+this.mRouteLegs[ this.mCurrentLeg ] );
    
    this.mRoute = this.mRoutes( this.mRouteLegs[ this.mCurrentLeg ] );
    this.mRoute.Go();
    
    if( this.mIsPaused || this.mIsAborted )
    {
      this.mHeading = gPlayer.MapLoc().head;
      break;
    }

    this.mCurrentLeg = this.mCurrentLeg+1;
  }
  
  gLog.TraceLine( "Lib.Action.Navigate.Navigate", "<< Action.Navigate.Navigate" );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_Turn()
{
  skapi.TurnToHead( this.mHeading );
  skapi.WaitEvent( 5000, wemSpecific, evidOnNavStop );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_CombatMode()
{
  if( this.mCombatMode != null )
  {
    gInteract.EnsureCombatMode( this.mCombatMode );
  }
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_SetRouteFileName( pRouteFileName )
{
  gLog.TraceLine( "Lib.Action.Navigate.SetRouteFileName", ">> Action.Navigate.SetRouteFileName( "+pRouteFileName+" )" );
  
  var vXml = XdocLoadFile( pRouteFileName );
  this.mRoutes = Xnav.DictSzXrouteFromXdoc( vXml );

  gLog.TraceLine( "Lib.Action.Navigate.SetRouteFileName", "<< Action.Navigate.SetRouteFileName" );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_SetRouteLegs( pRouteLegs )
{
  gLog.TraceLine( "Lib.Action.Navigate.RouteLegs", ">> Action.Navigate.RouteLegs( "+pRouteLegs+" )" );
  this.mRouteLegs = pRouteLegs.split( "," );
  gLog.TraceLine( "Lib.Action.Navigate.RouteLegs", "<< Action.Navigate.RouteLegs" );
}

//----------------------------------------------------------------------------------------------

function Action_Navigate_SetCombatMode( pCombatMode )
{
  if( pCombatMode == "true" || pCombatMode == "True" || pCombatMode == "TRUE" )
  {
    this.mCombatMode = true;
  }
  else
  {
    this.mCombatMode = false;
  }
}

//----------------------------------------------------------------------------------------------
