//----------------------------------------------------------------------------------------------
//
// Action Manager Object
//
//----------------------------------------------------------------------------------------------
//
// ...
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-09-07   Initial version.
//
//----------------------------------------------------------------------------------------------

function ActionManager()
{
  // Member variables
  this.mActionQueue = new ActionQueue();
  this.mMaxPriority = 100;
  this.mActions = new Dictionary();
  this.mCurrentAction = null;
  this.mIsIdle = false;
  this.mIdleTimer = skapi.TimerNew();

  // Member functions
  this.ReadConfigFromFile = ActionManager_ReadConfigFromFile;
  this.AddAction = ActionManager_AddAction;
  this.ExecuteAction = ActionManager_ExecuteAction;
  this.HasActiveAction = ActionManager_HasActiveAction;
  this.AbortAllActions = ActionManager_AbortAllActions;
  this.IdleTime = ActionManager_IdleTime;
  this.GetAction = ActionManager_GetAction;
}

//----------------------------------------------------------------------------------------------

function ActionManager_ReadConfigFromFile( pFileName )
{
  gLog.TraceLine( "Lib.ActionManager.ReadConfigFromFile", ">> ActionManager.ReadConfigFromFile( "+pFileName+" )" );

  var vConfig = new Config();
  var e, e2;

  vConfig.ReadFromFile( pFileName );

  e = vConfig.Enumerator( "//Actions/Action" );
  for( ; !e.atEnd(); e.moveNext() )
  {
    var vName = e.GetValueString( "Name" );
    var vClassName = e.GetValueString( "ClassName" );

    var vAction = eval( "new "+vClassName+"()" );

    gDynamic.Implements( vAction, gTemplates.Action );

    vAction.SetName( vName );
    vAction.SetPriority( this.mMaxPriority-- );

    e2 = e.Enumerator( "Parameters/Parameter" );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vParamName = e2.GetValueString( "Name" );
      var vParamValue = e2.GetValueString( "Value" );
      gDynamic.SetParameter( vAction, vParamName, vParamValue );
    }

    gLog.DebugLine( "Lib.ActionManager.ReadConfigFromFile", "Adding possible action: "+vName );

    vAction.SetActionManager( this );

    this.mActions.add( vName, vAction );

  }  

  gLog.TraceLine( "Lib.ActionManager.ReadConfigFromFile", "<< ActionManager.ReadConfigFromFile" );
}

//----------------------------------------------------------------------------------------------

function ActionManager_AddAction( pActionName )
{
  gLog.TraceLine( "Lib.ActionManager.AddAction", ">> ActionManager.AddAction( "+pActionName+" )" );

  var vAction = this.mActions( pActionName );

  gLog.DebugLine( "Lib.ActionManager.AddAction", "Found Action object? "+( vAction != null ) );

  var vActionItem = new ActionQueueItem( vAction, vAction.mPriority );

  this.mActionQueue.Add( vActionItem );

  this.mActionQueue.DebugInfo();

  if( this.mCurrentAction != null )
  {
    if( this.mCurrentAction.mPriority > vAction.mPriority )
    {
      gLog.InfoLine( "Ai.Action", "Pause Action: "+this.mCurrentAction.GetName() );
      this.mCurrentAction.PauseAction();
    }
  }

  gLog.TraceLine( "Lib.ActionManager.AddAction", "<< ActionManager.AddAction" );
}

//----------------------------------------------------------------------------------------------

function ActionManager_ExecuteAction()
{
  if( this.mActionQueue.IsEmpty() )
  {
    if( !this.mIsIdle )
    {
      gLog.DebugLine( "Lib.ActionManager.ExecuteAction", "ActionManager.ExecuteAction: Idle" );
      this.mIsIdle = true;
      this.mIdleTimer = skapi.TimerNew();
    }
    return;
  }

  this.mIsIdle = false;

  gLog.TraceLine( "Lib.ActionManager.ExecuteAction", ">> ActionManager.ExecuteAction()" );

  gPauseManager.SetActionManager( this );

  var vAction = this.mActionQueue.GetMin();

  this.mCurrentAction = vAction;

  if( vAction.IsPaused() )
  {
    gLog.InfoLine( "Ai.Action", "Resume Action: "+vAction.GetName() );
    vAction.ResumeAction();
  }
  else
  {
    gLog.InfoLine( "Ai.Action", "Start Action: "+vAction.GetName() );
    vAction.StartAction();
  }

  if( !vAction.IsPaused() && !this.mActionQueue.IsEmpty() )
  {
    this.mActionQueue.RemoveMin();
    gLog.InfoLine( "Ai.Action", "Finish Action: "+vAction.GetName() );
  }

  this.mCurrentAction = null;

  gLog.TraceLine( "Lib.ActionManager.ExecuteAction", "<< ActionManager.ExecuteAction" );
}

//----------------------------------------------------------------------------------------------

function ActionManager_HasActiveAction( pName )
{
  return this.mActionQueue.HasAction( pName );
}

//----------------------------------------------------------------------------------------------

function ActionManager_AbortAllActions()
{
  gLog.TraceLine( "Lib.ActionManager.AbortAllActions", ">> ActionManager.AbortAllActions()" );

  this.mActionQueue.DebugInfo();

  while( !this.mActionQueue.IsEmpty() )
  {
    this.mActionQueue.DebugInfo();
    var vAction = this.mActionQueue.GetMin();
    vAction.AbortAction();
    this.mActionQueue.RemoveMin();
  }

  this.mIsIdle = false;

  gLog.TraceLine( "Lib.ActionManager.AbortAllActions", "<< ActionManager.AbortAllActions" );
}

//----------------------------------------------------------------------------------------------

function ActionManager_IdleTime()
{
  if( this.mIsIdle )
  {
    return this.mIdleTimer.cmsec;
  }
  else
  {
    return 0;
  }
}

//----------------------------------------------------------------------------------------------

function ActionManager_GetAction( pActionName )
{
  return this.mActions( pActionName );
}

//----------------------------------------------------------------------------------------------
