//----------------------------------------------------------------------------------------------
//
// Action Queue Object
//
//----------------------------------------------------------------------------------------------
//
// Maintains list of actions that are currently executing.
//
// Based on code stolen from somewhere, don't remember from where any longer.
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-09-07   Initial version.
//
//----------------------------------------------------------------------------------------------

function ActionQueue()
{
  // Member variables
  this.mMaxSize = 100;
  this.mCurrentSize = 0;
  this.mArray = new Array( this.mMaxSize );

  // Member functions
  this.Add = ActionQueue_Add;
  this.RemoveAll = ActionQueue_RemoveAll;
  this.RemoveMin = ActionQueue_RemoveMin;
  this.GetMin = ActionQueue_GetMin;
  this.IsEmpty = ActionQueue_IsEmpty;
  this.HasAction = ActionQueue_HasAction;
  
  this.PercolateDown = ActionQueue_PercolateDown;

  this.DebugInfo = ActionQueue_DebugInfo;

}

//----------------------------------------------------------------------------------------------

function ActionQueueItem( pAction, pPriority )
{
  this.mAction = pAction;
  this.mPriority = pPriority;
}

//----------------------------------------------------------------------------------------------

function ActionQueue_Add( pActionQueueItem )
{
  gLog.TraceLine( "Lib.ActionQueue.Add", ">> ActionQueue.Add" );

  var vHole = ++this.mCurrentSize;
  
  for( ; vHole > 1 && pActionQueueItem.mPriority < this.mArray[ Math.floor( vHole/2 ) ].mPriority; vHole = Math.floor( vHole/2 ) )
  {
    this.mArray[ vHole ] = this.mArray[ Math.floor( vHole/2 ) ];
  }
  this.mArray[ vHole ] = pActionQueueItem;

  gLog.TraceLine( "Lib.ActionQueue.Add", "<< ActionQueue.Add" );
}

//----------------------------------------------------------------------------------------------

function ActionQueue_RemoveAll()
{
  this.mCurrentSize = 0;
}

//----------------------------------------------------------------------------------------------

function ActionQueue_IsEmpty()
{
  gLog.TraceLine( "Lib.ActionQueue.IsEmpty", ">> ActionQueue.IsEmpty()" );
  gLog.DebugLine( "Lib.ActionQueue.IsEmpty", "Current Size: "+this.mCurrentSize );

  var vIsEmpty = ( this.mCurrentSize <= 0 );

  gLog.TraceLine( "Lib.ActionQueue.IsEmpty", "<< ActionQueue.IsEmpty: "+vIsEmpty );
  return vIsEmpty;
}

//----------------------------------------------------------------------------------------------

function ActionQueue_RemoveMin()
{
  gLog.TraceLine( "Lib.ActionQueue.RemoveMin", ">> ActionQueue.RemoveMin" );

  if( this.isEmpty )
  {
    gLog.ErrorLine( "Lib.ActionQueue.RemoveMin", "Tried to remove minimum element when queue is empty." );
    return;
  }
  this.mArray[ 1 ] = this.mArray[ this.mCurrentSize-- ];
  this.PercolateDown( 1 );

  gLog.TraceLine( "Lib.ActionQueue.RemoveMin", "<< ActionQueue.RemoveMin" );
}

//----------------------------------------------------------------------------------------------

function ActionQueue_GetMin()
{
  if( this.IsEmpty() )
  {
    gLog.ErrorLine( "Lib.ActionQueue.GetMin", "Tried to get minimum element when queue is empty." );
    return;
  }
  return this.mArray[ 1 ].mAction;
}

//----------------------------------------------------------------------------------------------

function ActionQueue_PercolateDown( pHole )
{
  var vChild = 0;
  var vTemp = this.mArray[ pHole ];

  for( ; pHole*2 <= this.mCurrentSize; pHole = vChild )
  {
    vChild = pHole*2;
    if( vChild != this.mCurrentSize && this.mArray[ vChild+1 ].mPriority < this.mArray[ vChild ].mPriority )
    {
      vChild++;
    }
    if( this.mArray[ vChild ].mPriority < vTemp.mPriority )
    {
      this.mArray[ pHole ] = this.mArray[ vChild ];
    }
    else
    {
      break;
    }
  }
  this.mArray[ pHole ] = vTemp;
}

//----------------------------------------------------------------------------------------------

function ActionQueue_DebugInfo()
{
  var i;
  var s = "";

  for( i = 1; i <= this.mCurrentSize; i++ )
  {
    s = s+this.mArray[ i ].mAction.GetName()+", ";
  }
  gLog.DebugLine( "Lib.ActionQueue.Array", "Current Queue: "+s.substr( 0, s.length-2 ) );
}

//----------------------------------------------------------------------------------------------

function ActionQueue_HasAction( pName )
{
  var i;

  for( i = 1; i <= this.mCurrentSize; i++ )
  {
    if( this.mArray[ i ].mAction.GetName() == pName )
    {
      return true;
    }
  }
  return false;
}

//----------------------------------------------------------------------------------------------
