//----------------------------------------------------------------------------------------------
//
// Buff Manager Object
//
//----------------------------------------------------------------------------------------------
//
// Uses Nerfus Buffus 2 to buff the player.
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-08-31   Initial version.
//
//----------------------------------------------------------------------------------------------

function BuffManager()
{
  gLog.TraceLine( "Lib.BuffManager.New", ">> BuffManager.New" );

  // Member variables
  this.mIsWaiting = false;
  this.mProfileList = new ActiveXObject("Scripting.Dictionary");
  this.mCurrentProfile = null;
  this.mSpellFinishDelay = 0;

  // Member functions
  this.AddBuffProfile = BuffManager_AddBuffProfile;
  this.RemoveBuffProfile = BuffManager_RemoveBuffProfile;
  this.ReadConfigFromFile = BuffManager_ReadConfigFromFile;
  this.StartBuff = BuffManager_StartBuff;
  this.PauseBuff = BuffManager_PauseBuff;
  this.ResumeBuff = BuffManager_ResumeBuff;
  this.AbortBuff = BuffManager_AbortBuff;
  this.WaitForEndSpell = BuffManager_WaitForEndSpell;
  this.SetSpellFinishDelay = BuffManager_SetSpellFinishDelay;

  // Event functions
  this.OnSpellAdd = BuffManager_OnSpellAdd;

  // Register event listener
  skapi.AddHandler( evidOnSpellAdd, this );

  gLog.TraceLine( "Lib.BuffManager.New", "<< BuffManager.New" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_AddBuffProfile( pBuffProfile )
{
  gLog.TraceLine( "Lib.BuffManager.AddBuffProfile", ">> BuffManager.AddBuffProfile" );
  this.mProfileList.add( pBuffProfile.GetName(), pBuffProfile );
  gLog.TraceLine( "Lib.BuffManager.AddBuffProfile", "<< BuffManager.AddBuffProfile" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_RemoveBuffProfile( pBuffProfileName )
{
  gLog.TraceLine( "Lib.BuffManager.RemoveBuffProfile", ">> BuffManager.RemoveBuffProfile" );
  this.mProfileList.remove( pBuffProfileName );
  gLog.TraceLine( "Lib.BuffManager.RemoveBuffProfile", "<< BuffManager.RemoveBuffProfile" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_ReadConfigFromFile( pFileName )
{
  gLog.TraceLine( "Lib.BuffManager.ReadConfigFromFile", ">> BuffManager.ReadConfigFromFile" );

  var vConfig = new Config();
  var e, e2;

  vConfig.ReadFromFile( pFileName );

  this.SetSpellFinishDelay( vConfig.GetValueString( "//BuffConfig/SpellFinishDelay" ) );

  e = vConfig.Enumerator( "//BuffConfig/BuffProfiles/BuffProfile" );
  gLog.DebugLine( "Lib.BuffManager.ReadConfigFromFile", "Found "+e.mNodeList.length+" buff profiles" );
  for( ; !e.atEnd(); e.moveNext() )
  {
    var vName = e.GetValueString( "Name" );
    gLog.DebugLine( "Lib.BuffManager.ReadConfigFromFile", "Reading profile: "+vName );
    var vNbName = e.GetValueString( "NbProfileName" );
    var vLastSpell = e.GetValueString( "LastSpellName" );

    var vBuffProfile = new BuffProfile();
    vBuffProfile.SetName( vName );
    vBuffProfile.SetNbProfileName( vNbName );
    vBuffProfile.SetLastSpellName( vLastSpell );

    this.AddBuffProfile( vBuffProfile );
  }

  gLog.TraceLine( "Lib.BuffManager.ReadConfigFromFile", "<< BuffManager.ReadConfigFromFile" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_StartBuff( pProfileName )
{
  gLog.TraceLine( "Lib.BuffManager.StartBuff", ">> Lib.BuffManager.StartBuff" );

  if( this.mCurrentProfile != null )
  {
    throw "Buff is already running!";
  }

  this.mCurrentProfile = this.mProfileList( pProfileName );

  gInteract.PasteLine( "/nbuff "+this.mCurrentProfile.GetNbProfileName() );

  this.mLastSpellCast = false;
  this.WaitForEndSpell();

  gLog.TraceLine( "Lib.BuffManager.StartBuff", "<< Lib.BuffManager.StartBuff" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_PauseBuff()
{
  gLog.TraceLine( "Lib.BuffManager.PauseBuff", ">> Lib.BuffManager.PauseBuff" );

  this.mIsWaiting = false;

  gInteract.PasteLine( "/nbpause" );

  gLog.TraceLine( "Lib.BuffManager.PauseBuff", "<< Lib.BuffManager.PauseBuff" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_ResumeBuff()
{
  gLog.TraceLine( "Lib.BuffManager.ResumeBuff", ">> Lib.BuffManager.ResumeBuff" );

  if( !this.mLastSpellCast )
  {
    if( this.mCurrentProfile == null )
    {
      throw "No profile to resume!";
    }
    if( this.mIsWaiting )
    {
      throw "Profile is already running!";
    }

    gInteract.PasteLine( "/nbresume" );

    this.WaitForEndSpell();
  }

  gLog.TraceLine( "Lib.BuffManager.ResumeBuff", "<< Lib.BuffManager.ResumeBuff" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_AbortBuff()
{
  gLog.TraceLine( "Lib.BuffManager.AbortBuff", ">> Lib.BuffManager.AbortBuff" );

  this.mIsWaiting = false;
  this.mCurrentProfile = null;

  gInteract.PasteLine( "/nbabort" );

  gLog.TraceLine( "Lib.BuffManager.AbortBuff", "<< Lib.BuffManager.AbortBuff" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_WaitForEndSpell()
{
  gLog.TraceLine( "Lib.BuffManager.WaitForEndSpell", ">> BuffManager.WaitForEndSpell" );

  this.mIsWaiting = true;
  while( this.mIsWaiting && !this.mLastSpellCast )
  {
    skapi.WaitEvent( 100 );
  }

  // Let current animation finish
  // FIXME: Should use OnAnimationComplete?
  //skapi.WaitEvent( this.mSpellFinishDelay, wemFullTimeout );
  gInteract.WaitActionComplete( this.mSpellFinishDelay );

  gLog.TraceLine( "Lib.BuffManager.WaitForEndSpell", "<< BuffManager.WaitForEndSpell" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_OnSpellAdd( pSpell )
{
  gLog.TraceLine( "Lib.BuffManager.OnSpellAdd", ">> EVENT: BuffManager.OnSpellAdd" );
  if( this.mCurrentProfile != null )
  {
    if( pSpell.szName == this.mCurrentProfile.GetLastSpellName() )
    {
      this.mLastSpellCast = true;
      this.mCurrentProfile = null;
    }
  }
  gLog.TraceLine( "Lib.BuffManager.OnSpellAdd", "<< EVENT: BuffManager.OnSpellAdd" );
}

//----------------------------------------------------------------------------------------------

function BuffManager_SetSpellFinishDelay( pDelay )
{
  gLog.TraceLine( "Lib.BuffManager.SetSpellFinishDelay", ">> BuffManager.SetSpellFinishDelay" );
  this.mSpellFinishDelay = pDelay;
  gLog.TraceLine( "Lib.BuffManager.SetSpellFinishDelay", "<< BuffManager.SetSpellFinishDelay" );
}

//----------------------------------------------------------------------------------------------
