//----------------------------------------------------------------------------------------------
//
// LogHandler Object
//
//----------------------------------------------------------------------------------------------
//
// Object that contains functions for logging and setting log levels, based on
// log4j's structure
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-08-31   Initial version.
// 2.1   2003-01-04   Restructured to writer/filter/logger.
//
//----------------------------------------------------------------------------------------------
//
// FIXME:
//  - Only one writer for each logger?
//
//----------------------------------------------------------------------------------------------

function LogHandler()
{
//  gLog.TraceLine( "Lib.Logger.New", ">> Logger.New" );

  // Member variables
  this.mLogWriters = new Dictionary();
  this.mLogFilters = new Dictionary();
  this.mLoggers = new Array();

  // Constants
  this.DebugLevel = 1;
  this.TraceLevel = 2;
  this.InfoLevel = 3;
  this.WarnLevel = 4;
  this.ErrorLevel = 5;
  this.FatalLevel = 6;

  // Member functions
  this.AddLogWriter = LogHandler_AddLogWriter;
  this.RemoveLogWriter = LogHandler_RemoveLogWriter;
  this.AddLogFilter = LogHandler_AddLogFilter;
  this.RemoveLogFilter = LogHandler_RemoveLogFilter;
  this.AddLogger = LogHandler_AddLogger;

  this.ReadConfigFromFile = LogHandler_ReadConfigFromFile;

  this.WriteLine = LogHandler_WriteLine;
  this.DebugLine = LogHandler_DebugLine;
  this.TraceLine = LogHandler_TraceLine;
  this.InfoLine = LogHandler_InfoLine;
  this.WarnLine = LogHandler_WarnLine;
  this.ErrorLine = LogHandler_ErrorLine;
  this.FatalLine = LogHandler_FatalLine;

  this.LevelFromString = LogHandler_LevelFromString;

//  gLog.TraceLine( "Lib.Logger.New", "<< Logger.New" );
}

//----------------------------------------------------------------------------------------------

function LogHandler_AddLogWriter( pWriterName, pLogWriter )
{
  gLog.TraceLine( "Lib.Logger.AddLogWriter", ">> Logger.AddLogWriter( "+pWriterName+" )" );

  this.mLogWriters.add( pWriterName, pLogWriter );

  gLog.TraceLine( "Lib.Logger.AddLogWriter", "<< Logger.AddLogWriter" );
}

//----------------------------------------------------------------------------------------------

function LogHandler_RemoveLogWriter( pWriterName )
{
  gLog.TraceLine( "Lib.Logger.RemoveLogWriter", ">> Logger.RemoveLogWriter( "+pWriterName+" )" );

  this.mLogWriters.remove( pWriterName );

  gLog.TraceLine( "Lib.Logger.RemoveLogWriter", "<< Logger.RemoveLogWriter" );
}

//----------------------------------------------------------------------------------------------

function LogHandler_AddLogFilter( pFilterName, pLogFilter )
{
  gLog.TraceLine( "Lib.Logger.AddLogFilter", ">> Logger.AddLogFilter( "+pFilterName+" )" );

  this.mLogFilters.add( pFilterName, pLogFilter );

  gLog.TraceLine( "Lib.Logger.AddLogFilter", "<< Logger.AddLogFilter" );
}

//----------------------------------------------------------------------------------------------

function LogHandler_RemoveLogFilter( pFilterName )
{
  gLog.TraceLine( "Lib.Logger.RemoveLogFilter", ">> Logger.RemoveLogFilter( "+pFilterName+" )" );

  this.mLogFilters.remove( pFilterName );

  gLog.TraceLine( "Lib.Logger.RemoveLogFilter", "<< Logger.RemoveLogFilter" );
}

//----------------------------------------------------------------------------------------------

function LogHandler_AddLogger( pLogger )
{
  gLog.TraceLine( "Lib.Logger.AddLogger", ">> Logger.AddLogger()" );

  this.mLoggers.push( pLogger );

  gLog.TraceLine( "Lib.Logger.AddLogger", "<< Logger.AddLogger" );
}

//----------------------------------------------------------------------------------------------

function LogHandler_ReadConfigFromFile( pFileName )
{
  gLog.TraceLine( "Lib.Logger.ReadConfigFromFile", ">> Logger.ReadConfigFromFile( "+pFileName+" )" );

  var vConfig = new Config();
  var e, e2;

  vConfig.ReadFromFile( pFileName );

  e = vConfig.Enumerator( "//LogConfig/Writers/Writer" );
  for( ; !e.atEnd(); e.moveNext() )
  {
    var vName = e.GetValueString( "Name" );
    var vClass = e.GetValueString( "Class" );

    gLog.DebugLine( "Lib.Logger.ReadConfigFromFile", "Writer: "+vName );
    gLog.DebugLine( "Lib.Logger.ReadConfigFromFile", "Class: "+vClass );

    var vLogWriter = eval( "new "+vClass+"()" );

    gDynamic.Implements( vLogWriter, gTemplates.LogWriter );

    e2 = e.Enumerator( "Parameters/Parameter" );
    
    gLog.DebugLine( "Lib.Logger.ReadConfigFromFile", "NofParameters Found: "+e2.mNodeList.length );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vParamName = e2.GetValueString( "Name" );
      var vParamValue = e2.GetValueString( "Value" );
      gDynamic.SetParameter( vLogWriter, vParamName, vParamValue );
    }

    this.AddLogWriter( vName, vLogWriter );
  }


  e = vConfig.Enumerator( "//LogConfig/Filters/Filter" );
  for( ; !e.atEnd(); e.moveNext() )
  {
    var vName = e.GetValueString( "Name" );
    var vClass = e.GetValueString( "Class" );

    gLog.DebugLine( "Lib.Logger.ReadConfigFromFile", "Filter: "+vName );
    gLog.DebugLine( "Lib.Logger.ReadConfigFromFile", "Class: "+vClass );

    var vLogFilter = eval( "new "+vClass+"()" );

    gDynamic.Implements( vLogFilter, gTemplates.LogFilter );

    e2 = e.Enumerator( "Parameters/Parameter" );
    gLog.DebugLine( "Lib.Logger.ReadConfigFromFile", "NofParameters Found: "+e2.mNodeList.length );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vParamName = e2.GetValueString( "Name" );
      var vParamValue = e2.GetValueString( "Value" );
      gDynamic.SetParameter( vLogFilter, vParamName, vParamValue );
    }

    this.AddLogFilter( vName, vLogFilter );
  }


  e = vConfig.Enumerator( "//LogConfig/Loggers/Logger" );
  for( ; !e.atEnd(); e.moveNext() )
  {
    var vLogger = new Logger();

    var vWriterName = e.GetValueString( "Writer" );
    vLogger.SetWriter( this.mLogWriters( vWriterName ) );

    e2 = e.Enumerator( "Filters/Filter" );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vFilterName = e2.GetValueString( "" );
      vLogger.AddFilter( this.mLogFilters( vFilterName ) );
    }

    var vLevel = e.GetValueString( "Level" );
    vLogger.SetLevel( this.LevelFromString( vLevel ) );

    e2 = e.Enumerator( "Categories/Category" );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vCategory = e2.GetValueString( "" );
      vLogger.AddCategory( vCategory );
    }

    e2 = e.Enumerator( "Categories/Exclude/Category" );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vCategory = e2.GetValueString( "" );
      vLogger.AddExcludeCategory( vCategory );
    }

    this.AddLogger( vLogger );
  }


  gLog.TraceLine( "Lib.Logger.ReadConfigFromFile", "<< Logger.ReadConfigFromFile" );
}

//----------------------------------------------------------------------------------------------

function LogHandler_WriteLine( pCategory, pLevel, pString )
{
  var i;
  var vLogger;
 
  for( i = 0; i < this.mLoggers.length; i++ )
  {
    vLogger = this.mLoggers[ i ];
    vLogger.WriteLine( pCategory, pLevel, pString );
  }
}

//----------------------------------------------------------------------------------------------

function LogHandler_DebugLine( pCategory, pString )
{
  this.WriteLine( pCategory, this.DebugLevel, pString );
}

//----------------------------------------------------------------------------------------------

function LogHandler_TraceLine( pCategory, pString )
{
  this.WriteLine( pCategory, this.TraceLevel, pString );
}

//----------------------------------------------------------------------------------------------

function LogHandler_InfoLine( pCategory, pString )
{
  this.WriteLine( pCategory, this.InfoLevel, pString );
}

//----------------------------------------------------------------------------------------------

function LogHandler_WarnLine( pCategory, pString )
{
  this.WriteLine( pCategory, this.WarnLevel, pString );
}

//----------------------------------------------------------------------------------------------

function LogHandler_ErrorLine( pCategory, pString )
{
  this.WriteLine( pCategory, this.ErrorLevel, pString );
}

//----------------------------------------------------------------------------------------------

function LogHandler_FatalLine( pCategory, pString )
{
  this.WriteLine( pCategory, this.FatalLevel, pString );
}

//----------------------------------------------------------------------------------------------

function LogHandler_LevelFromString( pLogLevel )
{
  switch( pLogLevel )
  {
    case "DEBUG":
      return this.DebugLevel;
      break;
    case "TRACE":
      return this.TraceLevel;
      break;
    case "INFO":
      return this.InfoLevel;
      break;
    case "WARN":
      return this.WarnLevel;
      break;
    case "ERROR":
      return this.ErrorLevel;
      break;
    case "FATAL":
      return this.FatalLevel;
      break;
  }
}

//----------------------------------------------------------------------------------------------

