//----------------------------------------------------------------------------------------------
//
// Logger Object
//
//----------------------------------------------------------------------------------------------
//
// Object that contains functions for logging and setting log levels, based on
// log4j's structure
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-08-31   Initial version.
// 2.1   2003-01-04   Split logic into LogHandler and Logger classes.
//
//----------------------------------------------------------------------------------------------

function Logger()
{
  gLog.TraceLine( "Lib.Logger.New", ">> Logger.New" );

  // Member variables
  this.mWriter = null;
  this.mFilters = new Array();
  this.mLevel = 0;
  this.mCategories = new Array();
  this.mExclude = new Array();

  // Member functions
  this.SetWriter = Logger_SetWriter;
  this.AddFilter = Logger_AddFilter;
  this.SetLevel = Logger_SetLevel;
  this.AddCategory = Logger_AddCategory;
  this.AddExcludeCategory = Logger_AddExcludeCategory;
  this.WriteLine = Logger_WriteLine;
  this.IsSubdomain = Logger_IsSubdomain;

  gLog.TraceLine( "Lib.Logger.New", "<< Logger.New" );
}

//----------------------------------------------------------------------------------------------

function Logger_SetWriter( pWriter )
{
  this.mWriter = pWriter;
}

//----------------------------------------------------------------------------------------------

function Logger_AddFilter( pFilter )
{
  this.mFilters.push( pFilter );
}

//----------------------------------------------------------------------------------------------

function Logger_SetLevel( pLevel )
{
  this.mLevel = pLevel;
}

//----------------------------------------------------------------------------------------------

function Logger_AddCategory( pCategory )
{
  this.mCategories.push( pCategory );
}

//----------------------------------------------------------------------------------------------

function Logger_AddExcludeCategory( pCategory )
{
  this.mExclude.push( pCategory );
}

//----------------------------------------------------------------------------------------------

function Logger_WriteLine( pCategory, pLevel, pString )
{
  var vString;
  var i;
  var vFilter;

  // Log level high enuf for this writer to log the line?
  if( pLevel < this.mLevel )
  {
    return;
  }

  // Category excluded from logging?
  if( this.IsSubdomain( pCategory, this.mExclude ) )
  {
    return;
  }

  // Category included for logging?
  if( !this.IsSubdomain( pCategory, this.mCategories ) )
  {
    return;
  }

  // Apply all filters in order
  vString = pString;
  for( i = 0; i < this.mFilters.length; i++ )
  {
    vFilter = this.mFilters[ i ];
    vString = vFilter.ApplyFilter( pCategory, pLevel, vString );
    if( vString == "" )
    {
      return;
    }
  }

  this.mWriter.WriteLine( vString );
}

//----------------------------------------------------------------------------------------------

function Logger_IsSubdomain( pCategory, pCategories )
{
  var i;
  var vCategory;

  for( i = 0; i < pCategories.length; i++ )
  {
    vCategory = pCategories[ i ];
    if( pCategory.substr( 0, vCategory.length ) == vCategory )
    {
      return true;
    }
  }
  return false;
}

//----------------------------------------------------------------------------------------------
