//----------------------------------------------------------------------------------------------
//
// State - Stay In Place
//
//----------------------------------------------------------------------------------------------
//
// Will return character to position where state was entered after a certain idle time
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2003-02-10   Initial version.
//
//----------------------------------------------------------------------------------------------

function State_StayInPlace()
{
  // Member variables
  this.mSubStates = new Array();
  this.mIdleTime = 0;

  // Member functions
  this.EnterState = State_StayInPlace_EnterState;
  this.ExitState = State_StayInPlace_ExitState;
  this.OnTimer = State_StayInPlace_OnTimer;
  this.ReturnToMapLoc = State_StayInPlace_ReturnToMapLoc;
  this.SetMapLoc = State_StayInPlace_SetMapLoc;
  this.SetIdleTime = State_StayInPlace_SetIdleTime;
  this.SetDistance = State_StayInPlace_SetDistance;
  this.SetActionName = State_CorpseLooter_SetActionName;
  this.OnActionFailed = State_StayInPlace_OnActionFailed;
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_EnterState()
{
  gLog.TraceLine( "State.StayInPlace.EnterState", ">> State.StayInPlace.EnterState" );

  this.Inherited_EnterState();
  this.SetMapLoc( gPlayer.MapLoc() );
  gTimer.AddTimeout( "StayTimer", 300, this, true );

  gLog.TraceLine( "Lib.State.StayInPlace.EnterState", "<< State.StayInPlace.EnterState" );
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_ExitState()
{
  gLog.TraceLine( "State.StayInPlace.ExitState", ">> State.StayInPlace.ExitState" );

  gTimer.RemoveTimeout( "StayTimer" );
  this.Inherited_ExitState();

  gLog.TraceLine( "State.StayInPlace.ExitState", "<< State.StayInPlace.ExitState" );
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_OnTimer( pData )
{
  if( this.mActionManager.HasActiveAction( this.mActionName ) )
  {
    return;
  }

  if( this.mActionManager.IdleTime() < this.mIdleTime )
  {
    return;
  }
  
  if( gPlayer.MapLoc().Dist2DToMaploc( this.mMapLoc ) < this.mDistance )
  {
    return;
  }
  
  gLog.TraceLine( "State.StayInPlace.OnTimer", "Distance to loc: "+gPlayer.MapLoc().Dist2DToMaploc( this.mMapLoc ) );
  gLog.TraceLine( "State.StayInPlace.OnTimer", "Max distance: "+this.mDistance );
  
  this.ReturnToMapLoc();
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_ReturnToMapLoc()
{
  gLog.TraceLine( "State.StayInPlace.ExitState", ">> State.StayInPlace.ReturnToMapLoc()" );

  this.mActionManager.GetAction( this.mActionName ).SetMapLoc( this.mMapLoc );
  this.mActionManager.GetAction( this.mActionName ).SetDistance( this.mDistance );
  this.mActionManager.AddAction( this.mActionName );

  gLog.TraceLine( "State.StayInPlace.ExitState", "<< State.StayInPlace.ReturnToMapLoc" );
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_SetMapLoc( pMapLoc )
{
  this.mMapLoc = pMapLoc;
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_SetDistance( pDistance )
{
  this.mDistance = new Number( pDistance );
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_SetIdleTime( pIdleTime )
{
  this.mIdleTime = pIdleTime;
}

//----------------------------------------------------------------------------------------------

function State_CorpseLooter_SetActionName( pActionName )
{
  this.mActionName = pActionName;
}

//----------------------------------------------------------------------------------------------

function State_StayInPlace_OnActionFailed( pSource, pAction, pReason )
{
  // Do stuff here
}

//----------------------------------------------------------------------------------------------
