//----------------------------------------------------------------------------------------------
//
// State machine object
//
//----------------------------------------------------------------------------------------------
//
// Runs the state model
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-09-04   Initial version.
//
//----------------------------------------------------------------------------------------------
//
// FIXME:
//  - State transitions needs to be improved
//  - Each state shares a ActionManager currently, should they have separate?
//  - Handle start/end states in statemodel
//
//----------------------------------------------------------------------------------------------

function StateMachine()
{
  // Member variables
  this.mStateModel = null;
  this.mActionManager = null;

  this.mCurrentState = null;
  this.mNewStateName = "";

  // Member functions
  this.SetStateModel = StateMachine_SetStateModel;
  this.SetActionManager = StateMachine_SetActionManager;
  this.Execute = StateMachine_Execute;
  this.ExecuteState = StateMachine_ExecuteState;
  this.CurrentState = StateMachine_CurrentState;
  this.ChangeState = StateMachine_ChangeState;
  this.ShouldExitState = StateMachine_ShouldExitState;
}

//----------------------------------------------------------------------------------------------

function StateMachine_SetStateModel( pStateModel )
{
  this.mStateModel = pStateModel;
}

//----------------------------------------------------------------------------------------------

function StateMachine_SetActionManager( pActionManager )
{
  this.mActionManager = pActionManager;
}

//----------------------------------------------------------------------------------------------

function StateMachine_Execute( pStartStateName )
{
  gLog.TraceLine( "Lib.StateMachine.Execute", ">> StateMachine.Execute" );

  this.ExecuteState( pStartStateName );
  while( true )
  {
    gLog.DebugLine( "Lib.StateMachine.Execute", "StateMachine.Execute - Inside loop, beginning" );

    var vStateName = this.mNewStateName;
    this.mNewStateName = "";
    
    // FIXME:
    //  - Hardcoded endstate is bad, i need to know which endstate i came to

    if( vStateName == "End" )
    {
      break;
    }
    
    this.ExecuteState( vStateName );
  }

  gLog.TraceLine( "Lib.StateMachine.Execute", "<< StateMachine.Execute" );
}

//----------------------------------------------------------------------------------------------

function StateMachine_ExecuteState( pStateName )
{
  gLog.TraceLine( "Lib.StateMachine.ExecuteState", ">> StateMachine.ExecuteState( "+pStateName+" )" );

  var vState = this.mStateModel.mStates( pStateName );

  gLog.InfoLine( "Ai.State", "Execute State: "+pStateName );

  vState.SetActionManager( this.mActionManager );
  vState.SetStateMachine( this );

  this.mActionManager.AbortAllActions();
  this.mCurrentState = vState;
  vState.EnterState();
  vState.Execute();
  vState.ExitState();
  this.mActionManager.AbortAllActions();
  this.mCurrentState = null;

  gLog.TraceLine( "Lib.StateMachine.ExecuteState", "<< StateMachine.ExecuteState" );
}

//----------------------------------------------------------------------------------------------

function StateMachine_CurrentState()
{
  return this.mCurrentState;
}

//----------------------------------------------------------------------------------------------

function StateMachine_ChangeState( pStateTransition )
{
  gLog.TraceLine( "Lib.StateMachine.ChangeState", ">> StateMachine.ChangeState( "+pStateTransition+" )" );
  
  this.mNewStateName = this.mStateModel.GetNewState( this.mCurrentState.GetName(), pStateTransition );
  
  gLog.DebugLine( "Lib.StateMachine.ChangeState", "New state: "+this.mNewStateName );
  
  this.mActionManager.AbortAllActions();
  
  gLog.TraceLine( "Lib.StateMachine.ChangeState", "<< StateMachine.ChangeState" );
}

//----------------------------------------------------------------------------------------------

function StateMachine_ShouldExitState()
{
  if( this.mNewStateName != "" )
  {
    return true;
  }
  else
  {
    return false;
  }
}

//----------------------------------------------------------------------------------------------
