//----------------------------------------------------------------------------------------------
//
// State model object
//
//----------------------------------------------------------------------------------------------
//
// Handles all possible states and parameters for them.
//
//----------------------------------------------------------------------------------------------
//
// 2.0   2002-09-03   Initial version.
//
//----------------------------------------------------------------------------------------------
//
// FIXME:
//  - Handle start/end states
//
//----------------------------------------------------------------------------------------------

function StateModel()
{
  gLog.TraceLine( "Lib.StateModel.ReadConfigFromFile", ">> StateModel.New" );

  // Member variables
  this.mStates = new Dictionary();
  this.mTransitions = new Dictionary();

  // Member functions
  this.ReadConfigFromFile = StateModel_ReadConfigFromFile;
  this.AddState = StateModel_AddState;
  this.RemoveState = StateModel_RemoveState;
  this.GetNewState = StateModel_GetNewState;

  gLog.TraceLine( "Lib.StateModel.ReadConfigFromFile", "<< StateModel.New" );
}

//----------------------------------------------------------------------------------------------

function StateModel_AddState( pName, pStateObject )
{
  gLog.TraceLine( "Lib.StateModel.AddState", ">> StateModel.AddState( "+pName+" )" );

  this.mStates.add( pName, pStateObject );

  gLog.TraceLine( "Lib.StateModel.AddState", "<< StateModel.AddState" );
}

//----------------------------------------------------------------------------------------------

function StateModel_RemoveState( pName )
{
  gLog.TraceLine( "Lib.StateModel.RemoveState", ">> StateModel.RemoveState( "+pName+" )" );

  this.mStates.remove( pName );

  gLog.TraceLine( "Lib.StateModel.RemoveState", "<< StateModel.RemoveState" );
}

//----------------------------------------------------------------------------------------------

function StateModel_ReadConfigFromFile( pFileName )
{
  gLog.TraceLine( "Lib.StateModel.ReadConfigFromFile", ">> StateModel.ReadConfigFromFile( "+pFileName+" )" );

  var vConfig = new Config();
  var e, e2;

  vConfig.ReadFromFile( pFileName );

  e = vConfig.Enumerator( "//StateModel/States/State" );
  for( ; !e.atEnd(); e.moveNext() )
  {
    var vName = e.GetValueString( "Name" );
    var vClassName = e.GetValueString( "ClassName" );

    var vState = eval( "new "+vClassName+"()" );

    gDynamic.Implements( vState, gTemplates.State );

    vState.SetName( vName );

    e2 = e.Enumerator( "Parameters/Parameter" );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vParamName = e2.GetValueString( "Name" );
      var vParamValue = e2.GetValueString( "Value" );
      gDynamic.SetParameter( vState, vParamName, vParamValue );
    }

    e2 = e.Enumerator( "SubStates/SubState" );
    for( ; !e2.atEnd(); e2.moveNext() )
    {
      var vSubName = e2.GetValueString( "" );
      gLog.DebugLine( "Lib.StateModel.ReadConfigFromFile", "Substate: "+vSubName );
      vState.AddSubState( this.mStates( vSubName ) );
    }
    

    this.AddState( vName, vState );
  }

  e = vConfig.Enumerator( "//StateModel/Transitions/Transition" );
  for( ; !e.atEnd(); e.moveNext() )
  {
    var vFrom = e.GetValueString( "FromState" );
    var vName = e.GetValueString( "Name" );
    var vTo = e.GetValueString( "ToState" );

    this.mTransitions.add( vFrom+";"+vName, vTo );    
  }

  gLog.TraceLine( "Lib.StateModel.ReadConfigFromFile", "<< StateModel.ReadConfigFromFile" );
}

//----------------------------------------------------------------------------------------------

function StateModel_GetNewState( pState, pTransition )
{
  return this.mTransitions( pState+";"+pTransition );
}

//----------------------------------------------------------------------------------------------
